@@ -24,6 +24,22 @@
      &centroid       (sub {@_[0,1]}, @t));                                      # orange
   }
 
+ # An isosceles tringle with an apex height of 3/4 of the radius of its
+ # circumcircle divides Euler's line into 6 equal pieces
+
+ if (1)
+  {my $r = 400;                                                                 # Arbitrary but convenient radius
+   intersectionCircleLine                                                       # Find coordinates of equiangles of isoceles triangle
+    {my ($x, $y, $𝕩, $𝕪) = @_;                                                  # Coordinates of equiangles
+     my ($𝘅, $𝘆) = (0, $r);                                                     # Coordinates of apex
+     my ($nx, $ny, $nr) = ninePointCircle {@_} $x, $y, $𝘅, $𝘆, $𝕩, $𝕪;          # Coordinates of centre and radius of nine point circle
+     my ($cx, $cy)      = centroid        {@_} $x, $y, $𝘅, $𝘆, $𝕩, $𝕪;          # Coordinates of centroid
+     my ($ox, $oy)      = orthoCentre     {@_} $x, $y, $𝘅, $𝘆, $𝕩, $𝕪;          # Coordinates of orthocentre
+     ok join(' ', $y, $cy, $y+$nr, $𝘆, $y+2*$nr, $oy) eq                        # Calculated points along Euler's line
+        join(' ', 100, 200, 300, 400, 500, 600);                                # The actual six equally spaced points along Euler's line
+    } 0, 0, $r,  0, $r/4, 1, $r/4;                                              # Chord at 1/4 radius
+  }
+
  # A line segment across a circle is never longer than the diameter
 
  if (1)                                                                         # Random circle and random line
@@ -41,7 +57,7 @@
   {my ($x, $y, $r) = map {rand()} 1..3;                                         # Random circle
    my @p = intersectionCircles {@_} $x, $y, $r, $x+$r, $y, $r;                  # First step of one radius
  	 my @𝗽 = intersectionCircles {@_} $x, $y, $r, $p[0], $p[1], $r;               # Second step of one radius
- 	 my @q = !&near($x+$r, $y, @𝗽[0,1]) ? @𝗽[0,1] : @𝗽[2,3];                      # away from start point
+ 	 my @q = !&near($x+$r, $y, @𝗽[0,1]) ? @𝗽[0,1] : @𝗽[2,3];                      # Away from start point
  	 my @𝗾 = intersectionCircles {@_} $x, $y, $r, $q[0], $q[1], $r;               # Third step of one radius
    ok &near2(@𝗾[0,1], $x-$r, $y) or                                             # Brings us to a point
       &near2(@𝗾[2,3], $x-$r, $y);                                               # opposite to the start point
@@ -71,7 +87,8 @@
   }
 
 =cut
-
+# Since 1.005
+# 2015/12/21 Zero radius circle intersects with a line if the centre is collinear with the line
 package Math::Intersection::Circle::Line;
 #-------------------------------------------------------------------------------
 # Locate the points at which lines and circles cross in two dimensions
@@ -227,12 +244,12 @@ sub intersectionCircles(&$$$$$$)
       $x, $y, $r,                                                               # First circle centre, radius
       $𝘅, $𝘆, $𝗿) = @_;                                                         # Second circle centre, radius
   @_ == 7 or confess "Wrong number of parameters";
-  near($x, $𝘅) and near($y, $𝘆) and near($r, $𝗿) and                            # Complain if the two circles are in fact the same circle within the definition of nearness
-    confess "Duplicate circles!  Please provide two distinct circles.";
+  return &$sub("Duplicate circles!") if                                         # Complain if the two circles are in fact the same circle within the definition of nearness
+    near($x, $𝘅) and near($y, $𝘆) and near($r, $𝗿);
 
   my ($𝕏, $𝕐) = ($𝘅 - $x, $𝘆 - $y);                                             # Vector between centres
   my $𝗹 = vectorLength($𝕏, $𝕐);                                                 # Distance between centres
-  return &$sub() if $𝗹 > $r + $𝗿 or $𝗹 < abs($r - $𝗿);                          # The circles are too far apart or too close to intersect
+  return &$sub("No intersection!") if $𝗹 > $r + $𝗿 or $𝗹 < abs($r - $𝗿);         # The circles are too far apart or too close to intersect
 
   my ($𝕩, $𝕪) = ($𝕏 / $𝗹, $𝕐 / $𝗹);                                             # Unit vector between centres
   my $a = ($r*$r - $𝗿*$𝗿 + $𝗹*$𝗹)/ (2*$𝗹);                                      # Length of the common side
@@ -335,7 +352,8 @@ sub intersectionLines(&$$$$$$$$)
   @_ == 9 or confess "Wrong number of parameters";
   near($𝘅, $𝕩) and near($𝘆, $𝕪) and confess "Points on first line are too close!";
   near($𝗫, $𝕏) and near($𝗬, $𝕐) and confess "Points on second line are too close!";
-  near(atan2($𝘆-$𝕪, $𝘅-$𝕩), atan2($𝗬-$𝕐, $𝗫-$𝕏)) and confess "Parallel lines!"; # Lines are parallel if they have the same gradient
+  return &$sub("Parallel lines!") if                                             # Lines are parallel if they have the same gradient
+    near(atan2($𝘆-$𝕪, $𝘅-$𝕩), atan2($𝗬-$𝕐, $𝗫-$𝕏));
 
   intersectionLinePoint                                                         # Find 𝗮
    {my ($𝗮x, $𝗮y) = @_;
@@ -375,7 +393,10 @@ sub intersectionCircleLine(&$$$$$$$)
       $𝘅, $𝘆, $𝕩, $𝕪) = @_;                                                     # Line goes through these two points
   @_ == 8 or confess "Wrong number of parameters";
   near($𝘅, $𝕩) and near($𝘆, $𝕪) and confess "Points on line are too close!";
-  near($r) and confess "Radius is too small!";
+  if (near($r))                                                                 # Zero radius circle
+   {return &$sub($x, $y) if threeCollinearPoints($x, $y, $𝘅, $𝘆, $𝕩, $𝕪);       # Line passes through the centre of the circle
+    confess "Radius is too small!";
+   }
 
   intersectionLinePoint
    {my ($X, $Y) = @_;                                                           # Midpoint on line
@@ -387,8 +408,8 @@ sub intersectionCircleLine(&$$$$$$$)
      }
     my ($𝗫, $𝗬) = ($X - $x, $Y - $y);                                           # Centre to midpoint
     my $𝗗 = vectorLength($𝗫, $𝗬);                                               # Distance to midpoint
-    return &$sub()       if      $𝗗 > $r;                                       # Midpoint outside circle
-    return &$sub($X, $Y) if near($𝗗,  $r);                                      # Tangent
+    return &$sub("No intersection!") if $𝗗 > $r;                                # Midpoint outside circle
+    return &$sub($X, $Y)        if near($𝗗,  $r);                               # Tangent
     my $𝔻 = sqrt($r*$r - $𝗗*$𝗗);                                                # Length from midpoint along line to circumference
     my $s = $𝔻/$𝗗;                                                              # Length from midpoint along line to circumference relative to length from centre to midpoint
     &$sub($X - $s * $𝗬, $Y + $s * $𝗫, $X + $s * $𝗬, $Y - $s * $𝗫)               # Intersection points
@@ -478,9 +499,9 @@ sub circleInscribedInTriangle(&$$$$$$)
 
   intersectionLines
      {my ($X, $Y) = @_;                                                         # Intersection point
-      my @r = (unsignedDistanceFromLineToPoint(sub {@_}, $x, $y, $𝘅, $𝘆, $X, $Y),
-               unsignedDistanceFromLineToPoint(sub {@_}, $𝘅, $𝘆, $𝕩, $𝕪, $X, $Y),
-               unsignedDistanceFromLineToPoint(sub {@_}, $𝕩, $𝕪, $x, $y, $X, $Y));
+      my @r = ((unsignedDistanceFromLineToPoint {@_} $x, $y, $𝘅, $𝘆, $X, $Y),
+               (unsignedDistanceFromLineToPoint {@_} $𝘅, $𝘆, $𝕩, $𝕪, $X, $Y),
+               (unsignedDistanceFromLineToPoint {@_} $𝕩, $𝕪, $x, $y, $X, $Y));
       &near(@r[0,1]) && &near(@r[1,2]) or confess "Bad radius computed!";
       return &$sub($X, $Y, $r[0]);                                              # Coordinates of the centre of the inscribed circle, plus three estimates of its radius
      }
@@ -507,8 +528,60 @@ sub ninePointCircle(&$$$$$$)
     midPoint($𝕩, $𝕪, $x, $y));
  }
 
+#-------------------------------------------------------------------------------
+# Bisect the first angle of a triangle
+#-------------------------------------------------------------------------------
+
+sub bisectAnAngle(&$$$$$$)
+ {my ($sub, $x, $y, $𝘅, $𝘆, $𝕩, $𝕪) = @_;                                       # Subroutine to process results, coordinates of corners
+  @_ == 7 or confess "Wrong number of parameters";
+  (near($x, $𝘅) && near($y, $𝘆) or near($𝘅, $𝕩) && near($𝘆, $𝕪)) and confess "Corners are too close!";
+  my $𝕕 = vectorLength($x, $y, $𝕩, $𝕪);                                         # Lengths to opposite corners
+  my $𝗱 = vectorLength($x, $y, $𝘅, $𝘆);
+  &$sub($x, $y, $x + ($𝘅-$x)/$𝕕 + ($𝕩-$x)/$𝗱, $y + ($𝘆-$y)/$𝕕 + ($𝕪-$y)/$𝗱)     # Vector from vertex pointing along bisector
+ }
+
+#-------------------------------------------------------------------------------
+# 𝗙ind the centres and radii of the excircles of a triangle
+# https://en.wikipedia.org/wiki/Incircle_and_excircles_of_a_triangle
+# 𝗞nown: coordinates of each corner of the triangle
+# 𝗠ethod: intersection of appropriate angles of the triangles
+#-------------------------------------------------------------------------------
+
+sub exCircles(&$$$$$$)
+ {my ($sub, $x, $y, $𝘅, $𝘆, $𝕩, $𝕪) = @_;                                       # Subroutine to process results, coordinates of corners
+  @_ == 7 or confess "Wrong number of parameters";
+  (near($x, $𝘅) && near($y, $𝘆) or near($𝘅, $𝕩) && near($𝘆, $𝕪)) and confess "Corners are too close!";
+
+  my @c = &intersectionLines(sub{@_},                                           # Centres
+   (bisectAnAngle {@_} $x, $y, $𝘅, $𝘆,   $𝕩,        $𝕪),
+   (bisectAnAngle {@_} $𝘅, $𝘆, $𝕩, $𝕪, 2*$𝘅 - $x, 2*$𝘆 - $y));
+
+  my @𝗰 = &intersectionLines(sub{@_},
+   (bisectAnAngle {@_} $𝘅, $𝘆, $𝕩, $𝕪, $x, $y),
+   (bisectAnAngle {@_} $𝕩, $𝕪, $x, $y, 2*$𝕩 - $𝘅, 2*$𝕪 - $𝘆));
+
+  my @𝕔 = &intersectionLines(sub{@_},
+   (bisectAnAngle {@_} $𝕩, $𝕪, $x, $y, $𝘅, $𝘆),
+   (bisectAnAngle {@_} $x, $y, $𝘅, $𝘆, 2*$x - $𝕩, 2*$y - $𝕪));
+
+  my @r = (&unsignedDistanceFromLineToPoint(sub {@_}, $x, $y, $𝘅, $𝘆, @c),
+           &unsignedDistanceFromLineToPoint(sub {@_}, $𝘅, $𝘆, $𝕩, $𝕪, @c),
+           &unsignedDistanceFromLineToPoint(sub {@_}, $𝕩, $𝕪, $x, $y, @c));
+
+  my @𝗿 = (&unsignedDistanceFromLineToPoint(sub {@_}, $x, $y, $𝘅, $𝘆, @𝗰),
+           &unsignedDistanceFromLineToPoint(sub {@_}, $𝘅, $𝘆, $𝕩, $𝕪, @𝗰),
+           &unsignedDistanceFromLineToPoint(sub {@_}, $𝕩, $𝕪, $x, $y, @𝗰));
+
+  my @𝕣 = (&unsignedDistanceFromLineToPoint(sub {@_}, $x, $y, $𝘅, $𝘆, @𝕔),
+           &unsignedDistanceFromLineToPoint(sub {@_}, $𝘅, $𝘆, $𝕩, $𝕪, @𝕔),
+           &unsignedDistanceFromLineToPoint(sub {@_}, $𝕩, $𝕪, $x, $y, @𝕔));
+  ([@c, @r], [@𝗰, @𝗿], [@𝕔, @𝕣])                                                # For each circle, the centre followed by the radii estimates
+ }
+
 #-------------------------------------------------------------------------------
 # 𝗖entroid: intersection of lines between corners and mid points of opposite sides
+# 𝗙ind: coordinates of centroid
 # 𝗞nown: coordinates of each corner of the triangle
 #-------------------------------------------------------------------------------
 
@@ -523,7 +596,8 @@ sub centroid(&$$$$$$)
  }
 
 #-------------------------------------------------------------------------------
-# 𝗜ntersection of altitudes
+# 𝗢rthocentre: intersection of altitudes
+# 𝗙ind: coordinates of orthocentre
 # 𝗞nown: coordinates of each corner of the triangle
 #-------------------------------------------------------------------------------
 
@@ -621,19 +695,20 @@ use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);
 
 @ISA       = qw(Exporter);
 
-@EXPORT    = qw(intersectionCircles intersectionCirclesArea
+@EXPORT    = qw(exCircles intersectionCircles intersectionCirclesArea
 intersectionCircleLine intersectionCircleLineArea intersectionLines
 intersectionLinePoint circumCircle circumCentre circleInscribedInTriangle
-ninePointCircle areaOfTriangle areaOfPolygon  orthoCentre
-centroid isEquilateralTriangle isIsoscelesTriangle isRightAngledTriangle);
+ninePointCircle areaOfTriangle areaOfPolygon  orthoCentre centroid
+isEquilateralTriangle isIsoscelesTriangle isRightAngledTriangle);
 
 @EXPORT_OK = qw(midPoint near near2 near3 near4 rotate90CW rotate90CCW
 rotate90AroundMidPoint vectorLength 𝝿 lengthsOfTheSidesOfAPolygon
 threeCollinearPoints);
 
 $EXPORT_TAGS{all} = [@EXPORT, @EXPORT_OK];
-#$VERSION  = '1.003'; # Sunday 30 Aug 2015 - started Geometry app
-$VERSION   = '1.005'; # Sunday 20 Dec 2015 - still going!
+#$VERSION  = '1.003'; # Sun 30 Aug 2015 - started Geometry app
+#$VERSION  = '1.005'; # Sun 20 Dec 2015 - still going!
+$VERSION   = '1.006'; # Sat 02 Jan 2016 - Euler's line divided into 6 equal pieces
 
 =head1 Description
 
@@ -705,6 +780,20 @@ $VERSION   = '1.005'; # Sunday 20 Dec 2015 - still going!
  where (x, y), (𝘅, 𝘆) and (𝕩, 𝕪) are the coordinates of the vertices of the
  triangle.
 
+=head2 exCircles sub triangle
+
+ Calls sub([$x,$y,$r]...) where $x,$y are the coordinates of the centre of each
+ ex-circle and $r its radius for the specified triangle:
+
+ See: https://en.wikipedia.org/wiki/Incircle_and_excircles_of_a_triangle
+
+ A triangle is specified by supplying a list of six numbers:
+
+  (x, y, 𝘅, 𝘆, 𝕩, 𝕪)
+
+ where (x, y), (𝘅, 𝘆) and (𝕩, 𝕪) are the coordinates of the vertices of the
+ triangle.
+
 =head2 circleInscribedInTriangle sub triangle
 
  Calls sub($x,$y,$r) where $x,$y are the coordinates of the centre of
@@ -930,6 +1019,8 @@ $VERSION   = '1.005'; # Sunday 20 Dec 2015 - still going!
 
 =item C<circleThroughMidPointsOfTriangle()>
 
+=item C<exCircles()>
+
 =item C<intersectionCircleLine()>
 
 =item C<intersectionCircleLineArea()>
